/******************************************************************************

   ELEKTROMER

   LPC2132, 16.000 MHz

   (c) Ivo Strasil 2009, EGMedical, s.r.o.   www.strasil.net www.egmedical.cz

 *****************************************************************************/



#include <string.h>
#include <stdio.h>
#include <math.h>


#include "lpc_config.h"
#include "lpcUart.h"
#include "uart.h"

#include "wdt.h"
#include "interrupt_utils.h"

#include "kal_004.h"


#define BIT(x) ((unsigned long)1<<x)

static void systemInit(void)
{

	// --- enable and connect the PLL (Phase Locked Loop) ---
	// a. set multiplier and divider
	PLLCFG = MSEL | (1<<PSEL1) | (0<<PSEL0);
	// b. enable PLL
	PLLCON = (1<<PLLE);
	// c. feed sequence
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;
	// d. wait for PLL lock (PLOCK bit is set if locked)
	while (!(PLLSTAT & (1<<PLOCK)));
	// e. connect (and enable) PLL
	PLLCON = (1<<PLLE) | (1<<PLLC);
	// f. feed sequence
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;

	// --- setup and enable the MAM (Memory Accelerator Module) ---
	// a. start change by turning of the MAM (redundant)
	MAMCR = 0;
	// b. set MAM-Fetch cycle to 3 cclk as recommended for >40MHz
	MAMTIM = MAM_FETCH;
	// c. enable MAM
	MAMCR = MAM_MODE;

	// --- set VPB speed ---
	VPBDIV = VPBDIV_VAL;

	// --- map INT-vector ---
	#if defined(RAM_RUN)
	  MEMMAP = MEMMAP_USER_RAM_MODE;
	#elif defined(ROM_RUN)
	  MEMMAP = MEMMAP_USER_FLASH_MODE;
	#else
	#error RUN_MODE not defined!
	#endif
}


static void pllOff(void)
{

	// disable PLL
	PLLCON = 0;
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;

	// --- setup and enable the MAM (Memory Accelerator Module) ---
	// a. start change by turning of the MAM (redundant)
	MAMCR = 0;
/*	// b. set MAM-Fetch cycle to 3 cclk as recommended for >40MHz
	MAMTIM = MAM_FETCH;
	// c. enable MAM
	MAMCR = MAM_MODE;
*/

	// --- set VPB speed ---
	VPBDIV = 0;

	// --- map INT-vector ---
	#if defined(RAM_RUN)
	  MEMMAP = MEMMAP_USER_RAM_MODE;
	#elif defined(ROM_RUN)
	  MEMMAP = MEMMAP_USER_FLASH_MODE;
	#else
	#error RUN_MODE not defined!
	#endif
}



//prototyp preruseni
void __attribute__ ((interrupt("IRQ"))) ssp_int(void) ;


void ZprSSP(void);
void ZprSSP(void) {

}

//defines pro ovladani LED apod.

#define	LedOutOn()	IOSET1|=(1<<17);
#define	LedOutOff()	IOCLR1|=(1<<17);

#define	LedNoLoadOn()	IOSET1|=(1<<21);
#define	LedNoLoadOff()	IOCLR1|=(1<<21);

#define	LedRevOn()	IOSET1|=(1<<22);
#define	LedRevOff()	IOCLR1|=(1<<22);

#define	LedOlOn()	IOSET1|=(1<<19);
#define	LedOlOff()	IOCLR1|=(1<<19);


#define	OutOn()		IOSET1|=(1<<18);
#define	OutOff()	IOCLR1|=(1<<18);

#define	swNizsiProudy()		(IOPIN0&(1<<17))
#define	sw60V()				(IOPIN0&(1<<18))
#define	swZpPr()			(IOPIN0&(1<<19))

#define	swDC()				(IOPIN0&(1<<20))
#define	swMeritko1()		(IOPIN0&(1<<21))
#define	swMeritko2()		(IOPIN0&(1<<22))

#define	zpetnyProud() 	(IOPIN0&(1<<5))


//pomocne defines a funkce pro zapnuti preruseni

//zpozdeni behu, neni kalibrovane
void Delay(unsigned int x);
void Delay(unsigned int x) {
	for (volatile int i=0;i<x*1000;i++) {
		{}
	};
}



#define VICVectCntl0_ENABLE (1<<5)
#define VIC_Channel_TMR0  4
#define VIC_Channel_EINT2  16
void __attribute__ ((interrupt("IRQ"))) tmr0_int(void) ;
void __attribute__ ((interrupt("IRQ"))) eint2_int(void) ;


void tmr_init_int(void);
void tmr_init_int(void) {

	VICVectAddr0 = (unsigned long)tmr0_int;   // set interrupt vector in 0
	VICDefVectAddr = (unsigned long)tmr0_int;
	VICVectCntl0 = VICVectCntl0_ENABLE | VIC_Channel_TMR0; // use it for Timer 0 Interrupt:
	VICIntEnable |= (1<<VIC_Channel_TMR0);    // Enable Timer0 Interrupt
}


void eint2_init_int(void);
void eint2_init_int(void) {

	//edge rising interrupt
	EXTPOLAR = 4;
	EXTMODE = 4;

	//povol preruseni
	VICVectAddr1 = (unsigned long)eint2_int;   // set interrupt vector in 0
	VICDefVectAddr = (unsigned long)eint2_int;
	VICVectCntl1 = VICVectCntl0_ENABLE | VIC_Channel_EINT2; // use it for Timer 0 Interrupt:
	VICIntEnable |= (1<<VIC_Channel_EINT2);    // Enable Timer0 Interrupt
}



/* TMR0 Interrupt Handler (ISR) */

void tmr0_int(void)
{
	T0IR = 1;	//ACK preruseni

	// Clear interrupt flag
	VICVectAddr = 0;       // Acknowledge Interrupt (rough?)
}

/* Prenos dat z EINT2 handleru */
unsigned volatile int bylPuls = 0;
unsigned volatile int pocetPulsu = 0;
unsigned volatile int prenesPulsy = 0;


/* EINT2 Interrupt Handler (ISR) */

void eint2_int(void)
{

	if ((!zpetnyProud()) || (swZpPr())) {
		bylPuls = 1;	
		pocetPulsu++;
	}
	


	EXTINT |= 4;	//ACK preruseni
	
	// Clear interrupt flag
	VICVectAddr = 0;       // Acknowledge Interrupt (rough?)
}

unsigned int getPocetPulsu(void);
unsigned int getPocetPulsu(void) {
	unsigned int temp1,temp2;
	do {
		temp1 = pocetPulsu;
		temp2 = pocetPulsu;
	} while (temp1!=temp2);
	return temp1;	
}

unsigned int clrPocetPulsu(void);
unsigned int clrPocetPulsu(void) {
	VICIntEnable &=~ (1<<VIC_Channel_EINT2);    // Disable Interrupt
	asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");
	
	unsigned int temp1 = pocetPulsu;
	pocetPulsu = 0;
	
	VICIntEnable |= (1<<VIC_Channel_EINT2);    // Enable Interrupt
	
	return temp1;
}


void portsInit(void);
void portsInit(void) {
	IODIR1|= (1<<23)|(1<<22)|(1<<21)|(1<<19)|(1<<18)|(1<<17);
	IODIR0|= (1<<4)|(1<<6)|(1<<10)|(1<<12)|(1<<13)|(1<<16);
}



//nastaveni a reset MCP

void initMCP(void);
void initMCP(void) {
	//nastaveni konfigurace

	//GAIN
	//G0
	IOCLR0 = (1<<10);
	//G1
	IOCLR0 = (1<<13);

	//FREQ
	//F2 = 0, F1 = 1, F0 = 1
	//F2
	IOCLR0 = (1<<4);
	//F1
	IOSET0 = (1<<6);
	//F0
	IOSET1 = (1<<23);

	//HPF
	IOSET0 = (1<<12);
	
	IODIR0 &= ~(1<<15);
	IODIR0 &= ~(1<<5);


	//reset MCP
	Delay(10);
	IOCLR0 = (1<<16);
	Delay(100);
	IOSET0 = (1<<16);
	Delay(10);

}


#
char u[17]; 
#
char* itoa (signed long long n){
#
int i=0,j;
#
char s[17];
#

if (n<0) {
	n=-n;
	s[i++]='-';
}

#
 
#
 
#
do{
#
s[i++]=(char)( n%10+48 );
#
n-=n%10;
#
}
#
while((n/=10)>0);
#
for (j=0;j<i;j++)
#
u[i-1-j]=s[j];
#
 
#
u[j]='\0';
#
return u;
#
}

int main(void)
{


	systemInit();

	//inicializace LED atd.

	


	uart0Init(UART_BAUD(115200), UART_8N1, UART_FIFO_8); // setup the UART
	
	portsInit();
	initMCP();



	//spusteni casovace
/*

	T0MR0 = 1562;
	T0MR1 = 3125;
	//T0MR1 = 6250;
	T0MCR = 1|16;
	T0PR = 0;
	T0PC = 0;
	T0TC = 0;
	T0TCR = 1;
	uart0Puts("\nIRQinit\n");

	tmr_init_int();

*/
	//	IODIR0|=(1<<12);
	//	IODIR0|=(1<<16);

	WDTInit();
	WDTFeed();	//soucasne spusti IRQ


//	PINSEL0=0;
//	PINSEL1=0;
	
	


	uart0Puts("\nNormalni provoz.\n");

	PINSEL0|=(1<<31);
	
	eint2_init_int();
/*
		uart0Puts("SPI rezim...\n");
		IOCLR0=(1<<16);
		IOSET1=(1<<23); //CS
		IODIR0&=~(1<<5);//sdo je vstup
		
		IOCLR0=(1<<4);//clk do nuly
		
		Delay(100);
		IOSET0=(1<<16);
		
		//CS do nuly
		IOCLR1=(1<<23);
		
		//7--vystavit bit
		IOSET0=(1<<6);
		
		//takt CLK
		IOSET0=(1<<4);
		IOCLR0=(1<<4);


		//6--vystavit bit
		IOCLR0=(1<<6);
		
		//takt CLK
		IOSET0=(1<<4);
		IOCLR0=(1<<4);


		//5--vystavit bit
		IOSET0=(1<<6);
		
		//takt CLK
		IOSET0=(1<<4);
		IOCLR0=(1<<4);


		//4--vystavit bit
		IOCLR0=(1<<6);
		
		//takt CLK
		IOSET0=(1<<4);
		IOCLR0=(1<<4);


		//3---vystavit bit
		IOSET0=(1<<6);
		
		//takt CLK
		IOSET0=(1<<4);
		IOCLR0=(1<<4);

		//2---vystavit bit
		
		
		//takt CLK
		IOSET0=(1<<4);
		IOCLR0=(1<<4);
		//1--vystavit bit
		IOCLR0=(1<<6);
		
		//takt CLK
		IOSET0=(1<<4);
		IOCLR0=(1<<4);
		//0--vystavit bit
		
		
		//takt CLK
		IOSET0=(1<<4);
		IOCLR0=(1<<4);

		
		//CS do jednicky
		IOSET1=(1<<23);
		uart0Puts("SPI ok...\n");
*/

	LedOlOn();
	Delay(100);
	LedOlOff();

	unsigned int accCnt = 0;
	
	unsigned int kalVal;
	
	unsigned int noLoadCnt = 0;
	unsigned int poslBylPuls = 0;
	
	unsigned long long  val1sum = 0;
	unsigned long long  val2sum = 0;
	
	unsigned long long  nasobsum = 0;
	
	unsigned int val1max = 0;
	unsigned int val2max = 0;

	signed int val1dc = 0;
	signed int val2dc = 0;
	
	unsigned int pocetMereni = 0;

	while (1) {

		signed short val1,val2;
/*	
		//CS do nuly
		IOCLR1=(1<<23);
		
		//cekej na dr puls
		
	//	uart0Puts("p\n");
		
		
		//mame puls
		
	//	uart0Puts("P\n");
		
		//cekame na konec pulsu

		do {
		} while (!(IOPIN0&(1<<5)));
		do {
		} while (IOPIN0&(1<<5));
		
		//mame konec pulsu, muzeme zkusit vycist data
		
	//	uart0Puts("R\n");
		
		//puls do log. 1 na hodinach
		signed short i=16;
		
		val1=0;val2=0;
		
		do {
			i--;
			//takt CLK
			IOSET0=(1<<4);
			
			IOCLR0=(1<<4);
			
			asm("nop");
			asm("nop");
		
			//nacti val1
			if ((IOPIN0&(1<<5))) {
				val1|=(1<<i);
			}
			
			
			
		} while (i);


		i=16;
		do {
			i--;
			//takt CLK
			IOSET0=(1<<4);
			IOCLR0=(1<<4);
		
			//nacti val2
			if ((IOPIN0&(1<<5))) {
				val2|=(1<<i);
			}
			
		} while (i);
		
		
		
		//CS do jednicky
		IOSET1=(1<<23);		
		
		//Delay(1);
		
		
		//tisk hodnot val1, val2
		//val1=123;
	
		//val1=-123;
	
	
		// *********** Vypocet RMS hodnot ************
		
		if (pocetMereni > 12800) {
		
		
			//vypocet odmocnin
			val1sum = (signed long long)val1sum / pocetMereni;
			val2sum = (signed long long)val2sum / pocetMereni;
				
			
			nasobsum = (signed long long)nasobsum / pocetMereni;
		
			
			double f0 = sqrt(val1sum);
			double f00 = sqrt(val2sum);
			double f000 = sqrt(nasobsum);
			
			double f1 = sqrt(val1sum);
			double f2 = sqrt(val2sum);

			double f3 = (nasobsum);
			double f4 = 10000.0*(f3/(f1*f2));


			val1dc = (val1dc + pocetMereni*10) / pocetMereni;
			val2dc = (val2dc + pocetMereni*10 )/ pocetMereni;
		
			uart0Puts("************** ");
			
			uart0Puts(itoa(f1));
			//uart0Puts(", ");
			//uart0Puts(itoa(val1sum));
			uart0Puts(", ");
			uart0Puts(itoa(f2));
//			uart0Puts(", ");
			//uart0Puts(itoa(val2sum));
			uart0Puts("\n");

			uart0Puts(itoa(f3));
			uart0Puts(", ");
		//	uart0Puts(itoa((signed long)nasobsum));
		//	uart0Puts(", ");
			uart0Puts(itoa(f4));
			uart0Puts("\n");
			


			uart0Puts(itoa(val1max));
			uart0Puts(", ");
			uart0Puts(itoa(val2max));
			uart0Puts("\n");

			uart0Puts(itoa(val1dc));
			uart0Puts(", ");
			uart0Puts(itoa(val2dc));
			uart0Puts("\n");
			
			double u_mv = 1000.0 * f1;
			
			if (sw60V()) {
				u_mv= u_mv  /  96.258;
			} else {
				u_mv= u_mv  /  48.1931;
			}
			
			
			
			double i_ma = 1000.0 * f2;
			
			if (swNizsiProudy()) {
				i_ma = i_ma / 4384.090;
			} else {
				i_ma = i_ma /  2182.531;
			}

			
			uart0Puts("Napeti [mV]: ");
			uart0Puts(itoa(u_mv));
			uart0Puts("\nProud [mA]: ");
			uart0Puts(itoa(i_ma));
			uart0Puts("\nPF: ");
			uart0Puts(itoa(f4));
			
			
		
			val1sum=0;
			val2sum=0;
			val1max=0;
			val2max=0;
			val1dc =0;
			val2dc =0;
			nasobsum = 0;
			pocetMereni = 0;
		} else {
			//pricteni do sum - kvadraty
			val1sum += (val1 * val1);
			val2sum += (val2 * val2);
			
			//nasobsum = (signed long long)nasobsum + (signed long long)(val1 * val2);
						
			nasobsum += (val1 * val2);
		
			val1dc+=val1;
			val2dc+=val2;
			
			if (val1<0) val1=-val1;
			if (val2<0) val2=-val2;
			
			if (val1>val1max) val1max = val1;
			if (val2>val2max) val2max = val2;
			
			pocetMereni++;			
		}
	
	*/

	
	
	
	
	
		WDTFeed();
		
		//nastaveni MCP3939
		
		//DC rezim
		if (swDC()) {
			IOCLR0 = (1<<12);
		} else {
			IOSET0 = (1<<12);
		}

		//zesileni proudu
		if (swNizsiProudy()) {
			//G0
			IOSET0 = (1<<10);
			
			if (sw60V()) {
				kalVal = KAL_60V_NIZSI;			
			} else {
				kalVal = KAL_230V_NIZSI;			
			}
			
			
		} else {
			//G0
			IOCLR0 = (1<<10);
			
			if (sw60V()) {
				kalVal = KAL_60V_VYSSI;			
			} else {
				kalVal = KAL_230V_VYSSI;			
			}
		}

		//prepocet hodnot kalibrace podle meritka
		if (swMeritko2() /*switch 1*/) {
			if (swMeritko1()) /*switch 2*/ {
				//oba sepnute
				kalVal = 10* kalVal ; //10 imp/Wh
			} else {
				//sepnuty sw 1
				kalVal = 5* kalVal ; //20 imp/Wh				
			}
		} else {
			if (swMeritko1()) {
				//sepnuty sw 2
				kalVal = 2* kalVal ; //50 imp/Wh								
			}
		}



		if (zpetnyProud()) {
			LedRevOn();
		} else {
			LedRevOff();
		}

		if (/*(IOPIN0&(1<<15))*/bylPuls) {		
			if (noLoadCnt<100) noLoadCnt+=10;
			bylPuls=0;			
		} else {
			LedOlOff();
			if (noLoadCnt>5) noLoadCnt--;
			
		}
		
		
		if (noLoadCnt>10) {
			LedOlOn();
		} else {
			LedOlOff();
		}

		if (noLoadCnt<80) {
			LedNoLoadOn();
		} else {
			LedNoLoadOff();
		}
		
		
		unsigned int aPulsy = clrPocetPulsu();
		accCnt += aPulsy*1000;
		
		if (accCnt>kalVal && !poslBylPuls) {
			
			accCnt-=kalVal;
			
			LedOutOn();	
			OutOn();	
			accCnt = 0;
			
			poslBylPuls = 1;	//zamezi "sliti" dvou impulsu
			
		} else {
			LedOutOff();
			OutOff();	
			poslBylPuls = 0;
		}

	  	Delay(100);
	// Delay(3);


	}
}
